/**
 * \file pkomplex_main.cpp
 *   Demo program a perzisztens komplex osztály kipróbálásához
 *
 */


#include <iostream>
#include <fstream>
#include "pkomplex.h"

using std::cout;
using std::endl;

int main() {
  std::ofstream f1("f1.dat");
  PKomplex k1(3, 4), k2;
  // lustaságból csak az abszolút értéket írjuk ki:
  cout << "k1.abs():" << k1.abs() << endl;
  cout << "k2.abs():" << k2.abs() << endl;

  // kiírjuk
  k1.write(f1);
  f1.close();

  std::ifstream f2("f1.dat");

  // visszaolvassuk
  k2.read(f2);
  f2.close();
  cout << "k2.abs():" << k2.abs() << endl;

  // konverziók:
  // PKomplex -> komplex (kompatibilitás biztosítja)
  Komplex k0 = k2;
  cout << "k0.abs():" << k0.abs() << endl;

  // Komplex -> Pkomplex (konstruktor biztosítja)
  PKomplex kk = k0;
  cout << "kk.abs():" << kk.abs() << endl;

  // most próbáljunk egy tömböt kiírni és visszaolvasni.
  PKomplex t1[5];
  t1[0] = Komplex(3, 4);
  t1[1] = Komplex(6, 8);
  t1[2] = Komplex(5, 12);

  f1.open("f1.dat");
  for (int i = 0; i < 5; i++)
    t1[i].write(f1);

  PKomplex t2[5];
  f2.open("f1.dat");
  for (int i = 0; i < 5; i++)
    t2[i].read(f2);

  for (int i = 0; i < 5; i++)
    cout << "t2[" << i << "].abs():" << t2[i].abs() << endl;

  return 0;
}

