/**
 * \file iroda.h
 *
 * Irodai hierarchiát megvalósító osztályok
 *
 */
#ifndef IRODA_H
#define IRODA_H

#ifndef VIRT_OROKLES_NELKUL
 #define VIRTUAL virtual
#else
 #define VIRTUAL
#endif

#include <ctime>
#include "string5.h"

typedef int csop_t;             // csoport típusa

/**
 * Alkalmazott
 */
class Alkalmazott {
protected:
    String   nev;		        // név
    double   fizetes;			// fizetés
public:
    Alkalmazott(const char *n, double fiz) :nev(n), fizetes(fiz) {}
    const char* getNev() const { return nev.c_str(); }
    virtual void kiir() const;
};


/**
 * Csopotvezetõ
 */
class  CsopVez : VIRTUAL public Alkalmazott {
protected:
    csop_t   csoport;		    // csoport azon.
public:
    CsopVez(const char *n, double f, csop_t cs)
	    :Alkalmazott(n, f),     // alaposztály inicializálása
        csoport(cs)             // csoport inicializálás
    {}

    csop_t getcsoport() {       // csoport lekérdezése
        return csoport;
    }
};

/**
 * Határozott idejű alkalmazott
 */
class HatIdeju : VIRTUAL public Alkalmazott {
protected:
    time_t  ido;	            // szerződése lejár ekkor
public:
    HatIdeju(const char *n, double f, time_t t)
        :Alkalmazott(n, f),     // alaposztály inicializálása
        ido(t)                  // idő inicializálás
    {}
};

/**
 * Határozott idejű csoportvezetõ
 */
class HatIdCsV :public CsopVez, public HatIdeju {
public:
    HatIdCsV(const char* n, double f, csop_t cs, time_t t) :
#ifdef VIRT_OROKLESSEL
         Alkalmazott(n, f),
#endif
         CsopVez(n, f*2, cs),   // szándékosan más fizetést kap,
         HatIdeju(n, f*10, t)   // hogy látható legyen az adatduplikáció
    {}

    void kiir() const;
};

/**
 * Határozott idejű csoportvezetõ helyettes
 */
class HatIdCsVezH :public HatIdCsV {
    CsopVez& kit;               // kit helyettesít.
                                // Gázos megoldás, ha elõbb megszűnik az, akire hivatkozik!
public:
    HatIdCsVezH(HatIdCsV& kit, const char* n, double f, time_t t) :
#ifdef VIRT_OROKLESSEL
        Alkalmazott(n, f),
#endif
        HatIdCsV(n, f, kit.getcsoport(), t), // alaposztály
        kit(kit)              // kit helyettesít
    {}

    void kiir() const;
};
#endif // IRODA_H
