/**
 * Felhasználói felület és a modell kapcsolata
 * Boost lib demo
 *
 * CodeBloks-ban arra a könytárra kell állítani a boost globális változót,
 * ahova a boost (boost/include és boost/lib) telepítve van.
 *   (Settings->Global variables...)
 *
 */
#include <iostream>
#include "modell.h"
#include "felhfelulet.h"
#include <boost/signals2.hpp>
#include <boost/bind/bind.hpp>
#include <boost/optional/optional_io.hpp>

using std::cout;
using std::endl;
using std::cin;

/// Játék függvény a paramétarátaás bemutatásához
int osszead(int a, int b) { return a + b; }

int main() {
    Kapcsolo k1("k1");
    Gomb g1(boost::bind(&Kapcsolo::callBack, &k1)); /// Itt kötjük össze  a felhasználói felületet és a modellt.
                                                    /// A bind készít funktort a tagfüggvényből és az objektum címéből.

    cout << "Megyomjuk a gombot: " << endl;
    cin.ignore(100, '\n');
    g1.nyom();      // felületen megnyomták
    cout << "Megint megyomjuk a gombot: " << endl;
    cin.ignore(100, '\n');
    g1.nyom();      // újra
    cout << "Ismet megyomjuk a gombot: " << endl;
    cin.ignore(100, '\n');
    g1.nyom();      // újra
    cout << "Ismet megyomjuk a gombot: " << endl;
    cin.ignore(100, '\n');
    g1.nyom();      // újra


/// Eddig nem sokban különbözött a töbszörös örökléses megoldástól, csupán nem kellett e callback osztály.
/// A signal-slot azonban sokkal rugalmasabb. Itt csupán egy példa:

    std::cout << "\nMindent elpszor be-, majd kikapcsol.\n";
    Kapcsolo k2("k2");
    boost::signals2::signal<void ()> beki;
    beki.connect(0, boost::bind(&Kapcsolo::be, &k1));
    beki.connect(1, boost::bind(&Kapcsolo::ki, &k1));
    beki.connect(1, boost::bind(&Kapcsolo::ki, &k2));
    beki.connect(0, boost::bind(&Kapcsolo::be, &k2));

    beki();

/// Máik példa, ami azt mutatja, hogy könnyú paramétert is átadni:

    std::cout << "\nHogy megy a parameteratadas?\n";
    boost::signals2::signal<int (int, int)> ad;
    ad.connect(osszead);
    std::cout << "1+2="   << ad(1,2) << std::endl;
    std::cout << "10+100=" << ad(10,100) << std::endl;
    return 0;
}
