/*
 * \file polimorf.cpp
 *
 * Többszörös öröklés és a polimorf viselkedés,
 * valamint a dynamic_cast bemutatása.
 *
 * A dynamic_cast-hoz fontos, hogy a fordításkor legyen
 * engedélyezve a Run-Time Type Info (RTTI) generálása
 * (Vis.stud: /GR opció)
 * GCC-nél automatikus
 *
 */

#include <iostream>
#include <stdexcept>

/// Utasítást kiíró és végrehajtó makró
#define _(x)  std::cout << #x << std::endl; x

using std::cout;
using std::endl;

/// A osztáy.
/// (struct: mindene publikus)
struct A {
    void f()        { cout << "\t A::f()\n"; }
    void f(int)     { cout << "\t A::f(int)\n";  }
    virtual ~A() {} // csak ezért virt., hogy legyen RTTI infó
};

/// B osztáy.
/// (struct: mindene publikus)
struct B {
    void f(double)  { cout << "\t B::f(double)\n"; };
};


/// C osztáy.
/// (struct: mindene publikus)
struct C {
    void f(int*)  { cout << "\t C::f(int*)\n"; };
};

/// AB osztáy.
class AB :public A, public B {
public:
    // Többszörös öröklésnél az azonos nevű tagfüggvényekről a fordító feltételezi,
    // hogy teljesen eltérő viselkedést valósítanak meg, ezért egyik elérését sem
    // engedi automatikusan, hiába publikus az öröklés.
    using A::f;     // Így elérhetõk az A-beli f()-ek
    using B::f;     // Így elérhető a B-beli f()is
    void f() {
        cout << "\t AB::f()\n";
        A::f();
        f(1);       // A::f(int)
        f(1.2);     // B::f(double)
    }
};

/// Kiprobáljuk...
int main() {
    AB ab;

    cout << "Fuggvenyek probaja:" << endl;
_(    ab.A::f();    )
_(    ab.f(5);      )   // A::f(int)
_(    ab.f(6.3);    )   // B::f(double)
_(    ab.f();       )   // AB::f()

    cout << endl << "Upcast-nak mennie kell:" << endl;
_(    A *Ap = &ab;  )    // az upcast automaiikus
_(    B *Bp = &ab;  )    // az upcast automaiikus

    cout << endl << "Pointerek konvertalasa:" << endl;
_(    AB *ABp = dynamic_cast<AB*>(Ap);  )   // biztonságos down cast
    if (ABp != 0) {
        cout << "Sikeres volt a down cast" << endl;
_(      ABp->f();   )
    } else {
        cout << "A down cast nem sikerult" << endl;
    }

_(  C *Cp = dynamic_cast<C*>(Ap);       )   // ez nem fog menni
    if (Cp == 0) cout << "AB* nem konvertalhato C*-ra" << endl;

    cout << endl << "Referencia konvertalasa:" << endl;
    // Referencia konvertálásánál a hibajelzés kivétellel jön
    try {
_(      AB& ab2 = dynamic_cast<AB&>(*Ap);)
        cout << "Sikeres volt a down cast referenciara is:" << endl;
_(      ab2.f();    )
        cout << "Kiserlet C referenciara valo konvertziora:" << endl;
_(      C& c2 = dynamic_cast<C&>(*Ap);)
    } catch (std::exception& e) {
        cout << e.what() << endl;
    }
    return 0;
}
